# Copyright 2013 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


def _format_timestamp(component):
    try:
        return component['heartbeat_timestamp'].split(".", 2)[0]
    except (TypeError, KeyError):
        return ''


class ListAgent(neutronV20.ListCommand):
    """List agents."""

    resource = 'agent'
    list_columns = ['id', 'agent_type', 'host', 'availability_zone', 'alive',
                    'admin_state_up', 'binary']
    _formatters = {'heartbeat_timestamp': _format_timestamp}
    sorting_support = True

    def extend_list(self, data, parsed_args):
        for agent in data:
            if 'alive' in agent:
                agent['alive'] = ":-)" if agent['alive'] else 'xxx'


class ShowAgent(neutronV20.ShowCommand):
    """Show information of a given agent."""

    resource = 'agent'
    allow_names = False
    json_indent = 5


class DeleteAgent(neutronV20.DeleteCommand):
    """Delete a given agent."""

    resource = 'agent'
    allow_names = False


class UpdateAgent(neutronV20.UpdateCommand):
    """Updates the admin status and description for a specified agent."""

    resource = 'agent'
    allow_names = False

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state',
            action='store_false',
            help=_('Set admin state up of the agent to false.'))
        parser.add_argument(
            '--description',
            help=_('Description for the agent.'))

    def args2body(self, parsed_args):
        body = {'admin_state_up': parsed_args.admin_state}
        neutronV20.update_dict(parsed_args, body,
                               ['description'])
        return {self.resource: body}



# Copyright 2016 NEC Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_serialization import jsonutils
from oslo_utils import uuidutils
import yaml

from neutronclient.tests.functional import base


class TestCLIFormatter(base.ClientTestBase):

    def setUp(self):
        super(TestCLIFormatter, self).setUp()
        self.net_name = 'net-%s' % uuidutils.generate_uuid()
        self.addCleanup(self.neutron, 'net-delete %s' % self.net_name)

    def _create_net(self, fmt, col_attrs):
        params = ['-c %s' % attr for attr in col_attrs]
        params.append('-f %s' % fmt)
        params.append(self.net_name)
        param_string = ' '.join(params)
        return self.neutron('net-create', params=param_string)

    def test_net_create_with_json_formatter(self):
        result = self._create_net('json', ['name', 'admin_state_up'])
        self.assertDictEqual({'name': self.net_name,
                              'admin_state_up': True},
                             jsonutils.loads(result))

    def test_net_create_with_yaml_formatter(self):
        result = self._create_net('yaml', ['name', 'admin_state_up'])
        self.assertDictEqual({'name': self.net_name,
                              'admin_state_up': True},
                             yaml.load(result))

    def test_net_create_with_value_formatter(self):
        # NOTE(amotoki): In 'value' formatter, there is no guarantee
        # in the order of attribute, so we use one attribute in this test.
        result = self._create_net('value', ['name'])
        self.assertEqual(self.net_name, result.strip())

    def test_net_create_with_shell_formatter(self):
        result = self._create_net('shell', ['name', 'admin_state_up'])
        result_lines = set(result.strip().split('\n'))
        self.assertSetEqual(set(['name="%s"' % self.net_name,
                                 'admin_state_up="True"']),
                            result_lines)



#    Copyright 2017 FUJITSU LIMITED
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util
from oslo_log import log as logging

from neutronclient._i18n import _
from neutronclient.osc import utils as osc_utils


LOG = logging.getLogger(__name__)

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('router_id', 'Router', column_util.LIST_BOTH),
    ('subnet_id', 'Subnet', column_util.LIST_BOTH),
    ('flavor_id', 'Flavor', column_util.LIST_BOTH),
    ('admin_state_up', 'State', column_util.LIST_BOTH),
    ('status', 'Status', column_util.LIST_BOTH),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
)


def _get_common_parser(parser):
    parser.add_argument(
        '--description',
        metavar='<description>',
        help=_('Description for the VPN service'))
    parser.add_argument(
        '--subnet',
        metavar='<subnet>',
        help=_('Local private subnet (name or ID)'))
    parser.add_argument(
        '--flavor',
        metavar='<flavor>',
        help=_('Flavor for the VPN service (name or ID)'))
    admin_group = parser.add_mutually_exclusive_group()
    admin_group.add_argument(
        '--enable',
        action='store_true',
        help=_("Enable VPN service")
    )
    admin_group.add_argument(
        '--disable',
        action='store_true',
        help=_("Disable VPN service")
    )
    return parser


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if is_create:
        if 'project' in parsed_args and parsed_args.project is not None:
            attrs['tenant_id'] = osc_utils.find_project(
                client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
    if parsed_args.description:
        attrs['description'] = str(parsed_args.description)
    if parsed_args.subnet:
        _subnet_id = client_manager.network.find_subnet(
            parsed_args.subnet).id
        attrs['subnet_id'] = _subnet_id
    if parsed_args.flavor:
        _flavor_id = client_manager.network.find_flavor(
            parsed_args.flavor,
            ignore_missing=False
        ).id
        attrs['flavor_id'] = _flavor_id
    if parsed_args.enable:
        attrs['admin_state_up'] = True
    if parsed_args.disable:
        attrs['admin_state_up'] = False
    return attrs


class CreateVPNService(command.ShowOne):
    _description = _("Create an VPN service")

    def get_parser(self, prog_name):
        parser = super(CreateVPNService, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name for the VPN service'))
        parser.add_argument(
            '--router',
            metavar='ROUTER',
            required=True,
            help=_('Router for the VPN service (name or ID)'))
        osc_utils.add_project_owner_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        if parsed_args.name:
            attrs['name'] = str(parsed_args.name)
        if parsed_args.router:
            _router_id = self.app.client_manager.network.find_router(
                parsed_args.router).id
            attrs['router_id'] = _router_id
        obj = client.create_vpnservice({'vpnservice': attrs})['vpnservice']
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteVPNService(command.Command):
    _description = _("Delete VPN service(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteVPNService, self).get_parser(prog_name)
        parser.add_argument(
            'vpnservice',
            metavar='<vpn-service>',
            nargs='+',
            help=_('VPN service to delete (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        result = 0
        for vpn in parsed_args.vpnservice:
            try:
                vpn_id = client.find_resource(
                    'vpnservice', vpn, cmd_resource='vpnservice')['id']
                client.delete_vpnservice(vpn_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete VPN service with "
                            "name or ID '%(vpnservice)s': %(e)s"),
                          {'vpnservice': vpn, 'e': e})

        if result > 0:
            total = len(parsed_args.vpnservice)
            msg = (_("%(result)s of %(total)s vpn service failed "
                     "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListVPNService(command.Lister):
    _description = _("List VPN services that belong to a given project")

    def get_parser(self, prog_name):
        parser = super(ListVPNService, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        obj = client.list_vpnservices()['vpnservices']
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers, (utils.get_dict_properties(s, columns) for s in obj))


class SetVPNSercice(command.Command):
    _description = _("Set VPN service properties")

    def get_parser(self, prog_name):
        parser = super(SetVPNSercice, self).get_parser(prog_name)
        _get_common_parser(parser)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name for the VPN service'))
        parser.add_argument(
            'vpnservice',
            metavar='<vpn-service>',
            help=_('VPN service to modify (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager,
                                  parsed_args, is_create=False)
        if parsed_args.name:
            attrs['name'] = str(parsed_args.name)
        vpn_id = client.find_resource(
            'vpnservice', parsed_args.vpnservice,
            cmd_resource='vpnservice')['id']
        try:
            client.update_vpnservice(vpn_id, {'vpnservice': attrs})
        except Exception as e:
            msg = (_("Failed to set vpn service '%(vpn)s': %(e)s")
                   % {'vpn': parsed_args.vpnservice, 'e': e})
            raise exceptions.CommandError(msg)


class ShowVPNService(command.ShowOne):
    _description = _("Display VPN service details")

    def get_parser(self, prog_name):
        parser = super(ShowVPNService, self).get_parser(prog_name)
        parser.add_argument(
            'vpnservice',
            metavar='<vpn-service>',
            help=_('VPN service to display (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        vpn_id = client.find_resource(
            'vpnservice', parsed_args.vpnservice,
            cmd_resource='vpnservice')['id']
        obj = client.show_vpnservice(vpn_id)['vpnservice']
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return (display_columns, data)



# Copyright 2015 Rackspace Hosting Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import inspect
import sys
from unittest import mock


from neutronclient.common import extension
from neutronclient.neutron.v2_0.contrib import _fox_sockets as fox_sockets
from neutronclient import shell
from neutronclient.tests.unit import test_cli20


class CLITestV20ExtensionJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['fox_socket']

    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionJSON, self).setUp(plurals={'tags': 'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("_fox_sockets", fox_sockets)]
        return contrib

    def test_ext_cmd_loaded(self):
        neutron_shell = shell.NeutronShell('2.0')
        ext_cmd = {'fox-sockets-list': fox_sockets.FoxInSocketsList,
                   'fox-sockets-create': fox_sockets.FoxInSocketsCreate,
                   'fox-sockets-update': fox_sockets.FoxInSocketsUpdate,
                   'fox-sockets-delete': fox_sockets.FoxInSocketsDelete,
                   'fox-sockets-show': fox_sockets.FoxInSocketsShow}
        for cmd_name, cmd_class in ext_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            self.assertEqual(cmd_class, found[0])

    def test_ext_cmd_help_doc_with_extension_name(self):
        neutron_shell = shell.NeutronShell('2.0')
        ext_cmd = {'fox-sockets-list': fox_sockets.FoxInSocketsList,
                   'fox-sockets-create': fox_sockets.FoxInSocketsCreate,
                   'fox-sockets-update': fox_sockets.FoxInSocketsUpdate,
                   'fox-sockets-delete': fox_sockets.FoxInSocketsDelete,
                   'fox-sockets-show': fox_sockets.FoxInSocketsShow}
        for cmd_name, cmd_class in ext_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            found_factory = found[0]
            self.assertEqual(cmd_class, found_factory)
            self.assertTrue(found_factory.__doc__.startswith("[_fox_sockets]"))

    def test_delete_fox_socket(self):
        # Delete fox socket: myid.
        resource = 'fox_socket'
        cmd = fox_sockets.FoxInSocketsDelete(test_cli20.MyApp(sys.stdout),
                                             None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_update_fox_socket(self):
        # Update fox_socket: myid --name myname.
        resource = 'fox_socket'
        cmd = fox_sockets.FoxInSocketsUpdate(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname'],
                                   {'name': 'myname'})

    def test_create_fox_socket(self):
        # Create fox_socket: myname.
        resource = 'fox_socket'
        cmd = fox_sockets.FoxInSocketsCreate(test_cli20.MyApp(sys.stdout),
                                             None)
        name = 'myname'
        myid = 'myid'
        args = [name, ]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_fox_sockets(self):
        # List fox_sockets.
        resources = 'fox_sockets'
        cmd = fox_sockets.FoxInSocketsList(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_fox_pagination(self):
        resources = 'fox_sockets'
        cmd = fox_sockets.FoxInSocketsList(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_show_fox_socket(self):
        # Show fox_socket: --fields id --fields name myid.
        resource = 'fox_socket'
        cmd = fox_sockets.FoxInSocketsShow(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])


class CLITestV20ExtensionJSONAlternatePlurals(test_cli20.CLITestV20Base):
    class IPAddress(extension.NeutronClientExtension):
        resource = 'ip_address'
        resource_plural = '%ses' % resource
        object_path = '/%s' % resource_plural
        resource_path = '/%s/%%s' % resource_plural
        versions = ['2.0']

    class IPAddressesList(extension.ClientExtensionList, IPAddress):
        shell_command = 'ip-address-list'

    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionJSONAlternatePlurals, self).setUp()

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        ip_address = mock.Mock()
        ip_address.IPAddress = self.IPAddress
        ip_address.IPAddressesList = self.IPAddressesList
        contrib.return_value = [("ip_address", ip_address)]
        return contrib

    def test_ext_cmd_loaded(self):
        neutron_shell = shell.NeutronShell('2.0')
        ext_cmd = {'ip-address-list': self.IPAddressesList}
        for cmd_name, cmd_class in ext_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            self.assertEqual(cmd_class, found[0])

    def test_list_ip_addresses(self):
        # List ip_addresses.
        resources = 'ip_addresses'
        cmd = self.IPAddressesList(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)


class CLITestV20ExtensionJSONChildResource(test_cli20.CLITestV20Base):
    class Child(extension.NeutronClientExtension):
        parent_resource = 'parents'
        child_resource = 'child'
        resource = '%s_%s' % (parent_resource, child_resource)
        resource_plural = '%sren' % resource
        child_resource_plural = '%ren' % child_resource
        object_path = '/%s/%%s/%s' % (parent_resource, child_resource_plural)
        resource_path = '/%s/%%s/%s/%%s' % (parent_resource,
                                            child_resource_plural)
        versions = ['2.0']

    class ChildrenList(extension.ClientExtensionList, Child):
        shell_command = 'parent-child-list'

    class ChildShow(extension.ClientExtensionShow, Child):
        shell_command = 'parent-child-show'

    class ChildUpdate(extension.ClientExtensionUpdate, Child):
        shell_command = 'parent-child-update'

    class ChildDelete(extension.ClientExtensionDelete, Child):
        shell_command = 'parent-child-delete'

    class ChildCreate(extension.ClientExtensionCreate, Child):
        shell_command = 'parent-child-create'

    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionJSONChildResource, self).setUp()

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        child = mock.Mock()
        child.Child = self.Child
        child.ChildrenList = self.ChildrenList
        child.ChildShow = self.ChildShow
        child.ChildUpdate = self.ChildUpdate
        child.ChildDelete = self.ChildDelete
        child.ChildCreate = self.ChildCreate
        contrib.return_value = [("child", child)]
        return contrib

    def test_ext_cmd_loaded(self):
        neutron_shell = shell.NeutronShell('2.0')
        ext_cmd = {'parent-child-list': self.ChildrenList,
                   'parent-child-show': self.ChildShow,
                   'parent-child-update': self.ChildUpdate,
                   'parent-child-delete': self.ChildDelete,
                   'parent-child-create': self.ChildCreate}
        for cmd_name, cmd_class in ext_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            self.assertEqual(cmd_class, found[0])

    def test_client_methods_have_parent_id_arg(self):
        methods = (self.client.list_parents_children,
                   self.client.show_parents_child,
                   self.client.update_parents_child,
                   self.client.delete_parents_child,
                   self.client.create_parents_child)
        for method in methods:
            argspec = inspect.getargspec(method)
            self.assertIn("parent_id", argspec.args)



# Copyright 2016 Radware LTD.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _get_listener_id(client, listener_id_or_name):
    return neutronV20.find_resourceid_by_name_or_id(
        client, 'listener', listener_id_or_name)


def _get_pool_id(client, pool_id_or_name):
    return neutronV20.find_resourceid_by_name_or_id(
        client, 'pool', pool_id_or_name, cmd_resource='lbaas_pool')


def _add_common_args(parser, is_create=True):
    parser.add_argument(
        '--name',
        help=_('Name of the policy.'))
    parser.add_argument(
        '--description',
        help=_('Description of the policy.'))
    parser.add_argument(
        '--action',
        required=is_create,
        metavar='ACTION',
        type=utils.convert_to_uppercase,
        choices=['REJECT', 'REDIRECT_TO_POOL', 'REDIRECT_TO_URL'],
        help=_('Action type of the policy.'))
    parser.add_argument(
        '--redirect-pool',
        help=_('ID or name of the pool for REDIRECT_TO_POOL action type.'))
    parser.add_argument(
        '--redirect-url',
        help=_('URL for REDIRECT_TO_URL action type. '
               'This should be a valid URL string.'))
    parser.add_argument(
        '--position',
        type=int,
        help=_('L7 policy position in ordered policies list. '
               'This must be an integer starting from 1. '
               'Not specifying the position will place the policy '
               'at the tail of existing policies list.'))


def _common_args2body(client, parsed_args, is_create=True):
    if parsed_args.redirect_url:
        if parsed_args.action != 'REDIRECT_TO_URL':
            raise exceptions.CommandError(_('Action must be REDIRECT_TO_URL'))
    if parsed_args.redirect_pool:
        if parsed_args.action != 'REDIRECT_TO_POOL':
            raise exceptions.CommandError(_('Action must be REDIRECT_TO_POOL'))
        parsed_args.redirect_pool_id = _get_pool_id(
            client, parsed_args.redirect_pool)
    if (parsed_args.action == 'REDIRECT_TO_URL' and
            not parsed_args.redirect_url):
        raise exceptions.CommandError(_('Redirect URL must be specified'))
    if (parsed_args.action == 'REDIRECT_TO_POOL' and
            not parsed_args.redirect_pool):
        raise exceptions.CommandError(_('Redirect pool must be specified'))

    attributes = ['name', 'description',
                  'action', 'redirect_pool_id', 'redirect_url',
                  'position', 'admin_state_up']
    if is_create:
        parsed_args.listener_id = _get_listener_id(
            client, parsed_args.listener)
        attributes.extend(['listener_id', 'tenant_id'])
    body = {}
    neutronV20.update_dict(parsed_args, body, attributes)
    return {'l7policy': body}


class ListL7Policy(neutronV20.ListCommand):
    """LBaaS v2 List L7 policies that belong to a given listener."""

    resource = 'l7policy'
    shadow_resource = 'lbaas_l7policy'
    pagination_support = True
    sorting_support = True
    list_columns = [
        'id', 'name', 'action', 'redirect_pool_id', 'redirect_url',
        'position', 'admin_state_up', 'status'
    ]


class ShowL7Policy(neutronV20.ShowCommand):
    """LBaaS v2 Show information of a given L7 policy."""

    resource = 'l7policy'
    shadow_resource = 'lbaas_l7policy'


class CreateL7Policy(neutronV20.CreateCommand):
    """LBaaS v2 Create L7 policy."""

    resource = 'l7policy'
    shadow_resource = 'lbaas_l7policy'

    def add_known_arguments(self, parser):
        _add_common_args(parser)
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state_up',
            action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--listener',
            required=True,
            metavar='LISTENER',
            help=_('ID or name of the listener this policy belongs to.'))

    def args2body(self, parsed_args):
        return _common_args2body(self.get_client(), parsed_args)


class UpdateL7Policy(neutronV20.UpdateCommand):
    """LBaaS v2 Update a given L7 policy."""

    resource = 'l7policy'
    shadow_resource = 'lbaas_l7policy'

    def add_known_arguments(self, parser):
        _add_common_args(parser, is_create=False)
        utils.add_boolean_argument(
            parser, '--admin-state-up',
            help=_('Specify the administrative state of the policy'
                   ' (True meaning "Up").'))

    def args2body(self, parsed_args):
        return _common_args2body(self.get_client(), parsed_args, False)


class DeleteL7Policy(neutronV20.DeleteCommand):
    """LBaaS v2 Delete a given L7 policy."""

    resource = 'l7policy'
    shadow_resource = 'lbaas_l7policy'



# Copyright (C) 2013 Yahoo! Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
from io import StringIO
import logging
import os
import re
import sys
from unittest import mock

import fixtures
from keystoneauth1 import session
import testtools
from testtools import matchers

from neutronclient.common import clientmanager
from neutronclient.neutron.v2_0 import network
from neutronclient import shell as openstack_shell


DEFAULT_USERNAME = 'username'
DEFAULT_PASSWORD = 'password'
DEFAULT_TENANT_ID = 'tenant_id'
DEFAULT_TENANT_NAME = 'tenant_name'
DEFAULT_AUTH_URL = 'http://127.0.0.1:5000/v2.0/'
DEFAULT_TOKEN = '3bcc3d3a03f44e3d8377f9247b0ad155'
DEFAULT_URL = 'http://quantum.example.org:9696/'
DEFAULT_REGION = 'regionOne'
DEFAULT_ENDPOINT_TYPE = 'public'
DEFAULT_API_VERSION = '2.0'
DEFAULT_SERVICE_TYPE = 'network'
DEFAULT_SERVICE_NAME = 'neutron'
DEFAULT_RETRIES = 3
DEFAULT_TIMEOUT = 3.0


class ShellTest(testtools.TestCase):

    FAKE_ENV = {
        'OS_USERNAME': DEFAULT_USERNAME,
        'OS_PASSWORD': DEFAULT_PASSWORD,
        'OS_TENANT_ID': DEFAULT_TENANT_ID,
        'OS_TENANT_NAME': DEFAULT_TENANT_NAME,
        'OS_AUTH_URL': DEFAULT_AUTH_URL,
        'OS_REGION_NAME': None,
        'HTTP_PROXY': None,
        'http_proxy': None,
    }

    # Patch os.environ to avoid required auth info.
    def setUp(self):
        super(ShellTest, self).setUp()
        for var in self.FAKE_ENV:
            self.useFixture(
                fixtures.EnvironmentVariable(
                    var, self.FAKE_ENV[var]))

    def shell(self, argstr, check=False, expected_val=0):
        # expected_val is the expected return value after executing
        # the command in NeutronShell
        orig = (sys.stdout, sys.stderr)
        clean_env = {}
        _old_env, os.environ = os.environ, clean_env.copy()
        try:
            sys.stdout = StringIO()
            sys.stderr = StringIO()
            _shell = openstack_shell.NeutronShell('2.0')
            _shell.run(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(expected_val, exc_value.code)
        finally:
            stdout = sys.stdout.getvalue()
            stderr = sys.stderr.getvalue()
            sys.stdout.close()
            sys.stderr.close()
            sys.stdout, sys.stderr = orig
            os.environ = _old_env
        return stdout, stderr

    def test_run_unknown_command(self):
        self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
        stdout, stderr = self.shell('fake', check=True)
        self.assertFalse(stdout)
        self.assertIn("Unknown command ['fake']", stderr.strip())

    def test_help(self):
        required = 'usage:'
        help_text, stderr = self.shell('help')
        self.assertThat(
            help_text,
            matchers.MatchesRegex(required))

    def test_bash_completion(self):
        required = '.*os_user_domain_id.*'
        bash_completion, stderr = self.shell('bash-completion')
        self.assertThat(
            bash_completion,
            matchers.MatchesRegex(required))

    def test_help_on_subcommand(self):
        required = [
            '.*?^usage: .* quota-list']
        stdout, stderr = self.shell('help quota-list')
        for r in required:
            self.assertThat(
                stdout,
                matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_command(self):
        required = 'usage:'
        help_text, stderr = self.shell('help network-create')
        self.assertThat(
            help_text,
            matchers.MatchesRegex(required))

    def test_bash_completion_in_outputs_of_help_command(self):
        help_text, stderr = self.shell('help')
        completion_cmd = "bash-completion"
        completion_help_str = ("Prints all of the commands and options "
                               "for bash-completion.")
        self.assertIn(completion_cmd, help_text)
        self.assertIn(completion_help_str, help_text)

    def test_bash_completion_command(self):
        # just check we have some output
        required = [
            '.*--tenant_id',
            '.*help',
            '.*--dns-nameserver']
        help_text, stderr = self.shell('neutron bash-completion')
        for r in required:
            self.assertThat(help_text,
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_build_option_parser(self):
        neutron_shell = openstack_shell.NeutronShell('2.0')
        result = neutron_shell.build_option_parser('descr', '2.0')
        self.assertIsInstance(result, argparse.ArgumentParser)

    @mock.patch.object(openstack_shell.NeutronShell, 'run')
    def test_main_with_unicode(self, fake_shell):
        unicode_text = u'\u7f51\u7edc'
        argv = ['net-list', unicode_text, unicode_text]
        fake_shell.return_value = 0
        ret = openstack_shell.main(argv=argv)
        fake_shell.assert_called_once_with([u'net-list', unicode_text,
                                           unicode_text])
        self.assertEqual(0, ret)

    def test_endpoint_option(self):
        shell = openstack_shell.NeutronShell('2.0')
        parser = shell.build_option_parser('descr', '2.0')

        # Neither $OS_ENDPOINT_TYPE nor --os-endpoint-type
        namespace = parser.parse_args([])
        self.assertEqual('public', namespace.os_endpoint_type)

        # --endpoint-type but not $OS_ENDPOINT_TYPE
        namespace = parser.parse_args(['--os-endpoint-type=admin'])
        self.assertEqual('admin', namespace.os_endpoint_type)

    def test_endpoint_environment_variable(self):
        fixture = fixtures.EnvironmentVariable("OS_ENDPOINT_TYPE",
                                               "public")
        self.useFixture(fixture)

        shell = openstack_shell.NeutronShell('2.0')
        parser = shell.build_option_parser('descr', '2.0')

        # $OS_ENDPOINT_TYPE but not --endpoint-type
        namespace = parser.parse_args([])
        self.assertEqual("public", namespace.os_endpoint_type)

        # --endpoint-type and $OS_ENDPOINT_TYPE
        namespace = parser.parse_args(['--endpoint-type=admin'])
        self.assertEqual('admin', namespace.endpoint_type)

    def test_timeout_option(self):
        shell = openstack_shell.NeutronShell('2.0')
        parser = shell.build_option_parser('descr', '2.0')

        # Neither $OS_ENDPOINT_TYPE nor --endpoint-type
        namespace = parser.parse_args([])
        self.assertIsNone(namespace.http_timeout)

        # --endpoint-type but not $OS_ENDPOINT_TYPE
        namespace = parser.parse_args(['--http-timeout=50'])
        self.assertEqual(50, namespace.http_timeout)

    def test_timeout_environment_variable(self):
        fixture = fixtures.EnvironmentVariable("OS_NETWORK_TIMEOUT",
                                               "50")
        self.useFixture(fixture)

        shell = openstack_shell.NeutronShell('2.0')
        parser = shell.build_option_parser('descr', '2.0')

        namespace = parser.parse_args([])
        self.assertEqual(50, namespace.http_timeout)

    def test_run_incomplete_command(self):
        self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
        cmd = (
            '--os-username test --os-password test --os-project-id test '
            '--os-auth-strategy keystone --os-auth-url '
            '%s port-create' %
            DEFAULT_AUTH_URL)
        stdout, stderr = self.shell(cmd, check=True, expected_val=2)
        search_str = "Try 'neutron help port-create' for more information"
        self.assertTrue(any(search_str in string for string
                            in stderr.split('\n')))

    def _test_authenticate_user(self, expect_verify, expect_insecure,
                                **options):
        base_options = {'os_cloud': None,
                        'http_timeout': DEFAULT_TIMEOUT,
                        'region_name': DEFAULT_REGION,
                        'network_service_name': DEFAULT_SERVICE_NAME,
                        'neutron_service_type': DEFAULT_SERVICE_TYPE}

        options.update(base_options)
        if options.get('os_token'):
            options.update({'auth_type': 'token'})
            options.update({'os_token': 'token', 'os_url': 'url'})
        else:
            options.update({'os_token': None, 'os_url': None})

        with mock.patch.object(openstack_shell.NeutronShell,
                               'run_subcommand'), \
                mock.patch.object(session, 'Session') as session_mock, \
                mock.patch.object(clientmanager, 'ClientManager') as cmgr_mock:

            shell = openstack_shell.NeutronShell(DEFAULT_API_VERSION)
            shell.options = mock.Mock(spec=options.keys())
            for k, v in options.items():
                setattr(shell.options, k, v)
            shell.options.os_endpoint_type = DEFAULT_ENDPOINT_TYPE
            shell.options.retries = DEFAULT_RETRIES

            if not (options.get('os_token') and options.get('os_url')):
                auth = mock.ANY
                auth_session = mock.sentinel.session
                session_mock.return_value = auth_session
            else:
                auth = None
                auth_session = None

            shell.authenticate_user()

        if not (options.get('os_token') and options.get('os_url')):
            session_mock.assert_called_once_with(
                auth=mock.ANY, verify=expect_verify,
                cert=options.get('cert'),
                timeout=DEFAULT_TIMEOUT)
        else:
            self.assertFalse(session_mock.called)

        cmgr_mock.assert_called_once_with(
            retries=DEFAULT_RETRIES,
            raise_errors=False,
            session=auth_session,
            url=options.get('os_url'),
            token=options.get('os_token'),
            region_name=DEFAULT_REGION,
            api_version=DEFAULT_API_VERSION,
            service_type=DEFAULT_SERVICE_TYPE,
            service_name=DEFAULT_SERVICE_NAME,
            endpoint_type=DEFAULT_ENDPOINT_TYPE,
            auth=auth,
            insecure=expect_insecure,
            log_credentials=True)

    def test_authenticate_secure_with_cacert_with_cert(self):
        self._test_authenticate_user(
            insecure=False, cacert='cacert', cert='cert',
            expect_verify='cacert', expect_insecure=False)

    def test_authenticate_secure_with_cacert_with_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=False, cacert='cacert', cert='cert',
            expect_verify='cacert', expect_insecure=False)

    def test_authenticate_insecure_with_cacert_with_cert(self):
        self._test_authenticate_user(
            insecure=True, cacert='cacert', cert='cert',
            expect_verify=False, expect_insecure=True)

    def test_authenticate_insecure_with_cacert_with_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=True, cacert='cacert', cert='cert',
            expect_verify=False, expect_insecure=True)

    def test_authenticate_secure_without_cacert_with_cert(self):
        self._test_authenticate_user(
            insecure=False, cert='cert',
            expect_verify=True, expect_insecure=False)

    def test_authenticate_secure_without_cacert_with_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=False, cert='cert',
            expect_verify=True, expect_insecure=False)

    def test_authenticate_insecure_without_cacert_with_cert(self):
        self._test_authenticate_user(
            insecure=True, cert='cert',
            expect_verify=False, expect_insecure=True)

    def test_authenticate_insecure_without_cacert_with_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=True, cert='cert',
            expect_verify=False, expect_insecure=True)

    def test_authenticate_secure_with_cacert_without_cert(self):
        self._test_authenticate_user(
            insecure=False, cacert='cacert',
            expect_verify='cacert', expect_insecure=False)

    def test_authenticate_secure_with_cacert_without_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=False, cacert='cacert',
            expect_verify='cacert', expect_insecure=False)

    def test_authenticate_insecure_with_cacert_without_cert(self):
        self._test_authenticate_user(
            insecure=True, cacert='cacert',
            expect_verify=False, expect_insecure=True)

    def test_authenticate_insecure_with_cacert_without_cert_with_token(self):
        self._test_authenticate_user(
            os_token='token',
            insecure=True, cacert='cacert',
            expect_verify=False, expect_insecure=True)

    def test_commands_dict_populated(self):
        # neutron.shell.COMMANDS is populated once NeutronShell is initialized.
        # To check COMMANDS during NeutronShell initialization,
        # reset COMMANDS to some dummy value before calling NeutronShell().
        self.useFixture(fixtures.MockPatchObject(openstack_shell,
                                                 'COMMANDS', None))
        openstack_shell.NeutronShell('2.0')
        self.assertDictContainsSubset(
            {'net-create': network.CreateNetwork,
             'net-delete': network.DeleteNetwork,
             'net-list': network.ListNetwork,
             'net-show': network.ShowNetwork,
             'net-update': network.UpdateNetwork},
            openstack_shell.COMMANDS['2.0'])



# Copyright 2014 Blue Box Group, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.neutron.v2_0.lb.v2 import loadbalancer as lb
from neutronclient.tests.unit import test_cli20


class CLITestV20LbLoadBalancerJSON(test_cli20.CLITestV20Base):

    def test_create_loadbalancer_with_mandatory_params(self):
        # lbaas-loadbalancer-create with mandatory params only.
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        cmd = lb.CreateLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        name = 'lbaas-loadbalancer-name'
        vip_subnet_id = 'vip-subnet'
        my_id = 'my-id'
        args = [vip_subnet_id]
        position_names = ['vip_subnet_id']
        position_values = [vip_subnet_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_create_loadbalancer_with_all_params(self):
        # lbaas-loadbalancer-create with all params set.
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        cmd = lb.CreateLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        name = 'lbaas-loadbalancer-name'
        description = 'lbaas-loadbalancer-desc'
        flavor_id = 'lbaas-loadbalancer-flavor'
        vip_subnet_id = 'vip-subnet'
        my_id = 'my-id'
        args = ['--admin-state-down', '--description', description,
                '--name', name, '--flavor', flavor_id, vip_subnet_id]
        position_names = ['admin_state_up', 'description', 'name',
                          'flavor_id', 'vip_subnet_id']
        position_values = [False, description, name, flavor_id, vip_subnet_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource)

    def test_list_loadbalancers(self):
        # lbaas-loadbalancer-list.
        resources = 'loadbalancers'
        cmd_resources = 'lbaas_loadbalancers'
        cmd = lb.ListLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True,
                                  cmd_resources=cmd_resources)

    def test_list_loadbalancers_pagination(self):
        # lbaas-loadbalancer-list with pagination.
        resources = 'loadbalancers'
        cmd_resources = 'lbaas_loadbalancers'
        cmd = lb.ListLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd,
                                                  cmd_resources=cmd_resources)

    def test_list_loadbalancers_sort(self):
        # lbaas-loadbalancer-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = 'loadbalancers'
        cmd_resources = 'lbaas_loadbalancers'
        cmd = lb.ListLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"],
                                  cmd_resources=cmd_resources)

    def test_list_loadbalancers_limit(self):
        # lbaas-loadbalancer-list -P.
        resources = 'loadbalancers'
        cmd_resources = 'lbaas_loadbalancers'
        cmd = lb.ListLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000,
                                  cmd_resources=cmd_resources)

    def test_show_loadbalancer_id(self):
        # lbaas-loadbalancer-loadbalancer-show test_id.
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        cmd = lb.ShowLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource)

    def test_show_loadbalancer_id_name(self):
        # lbaas-loadbalancer-loadbalancer-show.
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        cmd = lb.ShowLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=cmd_resource)

    def _test_update_lb(self, args, expected_values):
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        my_id = 'myid'
        args.insert(0, my_id)
        cmd = lb.UpdateLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, my_id,
                                   args, expected_values,
                                   cmd_resource=cmd_resource)

    def test_update_loadbalancer(self):
        # lbaas-loadbalancer-update myid --name newname.
        self._test_update_lb(['--name', 'newname'], {'name': 'newname', })
        # lbaas-loadbalancer-update myid --description check.
        self._test_update_lb(['--description', 'check'],
                             {'description': 'check', })
        # lbaas-loadbalancer-update myid --admin-state-up False.
        self._test_update_lb(['--admin-state-up', 'False'],
                             {'admin_state_up': 'False', })

    def test_delete_loadbalancer(self):
        # lbaas-loadbalancer-loadbalancer-delete my-id.
        resource = 'loadbalancer'
        cmd_resource = 'lbaas_loadbalancer'
        cmd = lb.DeleteLoadBalancer(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args,
                                   cmd_resource=cmd_resource)

    def test_retrieve_loadbalancer_stats(self):
        # lbaas-loadbalancer-stats test_id.
        resource = 'loadbalancer'
        cmd = lb.RetrieveLoadBalancerStats(test_cli20.MyApp(sys.stdout), None)
        my_id = self.test_id
        fields = ['bytes_in', 'bytes_out']
        args = ['--fields', 'bytes_in', '--fields', 'bytes_out', my_id]

        query = "&".join(["fields=%s" % field for field in fields])
        expected_res = {'stats': {'bytes_in': '1234', 'bytes_out': '4321'}}
        resstr = self.client.serialize(expected_res)
        path = getattr(self.client, "lbaas_loadbalancer_path_stats")
        return_tup = (test_cli20.MyResp(200), resstr)

        cmd_parser = cmd.get_parser("test_" + resource)
        parsed_args = cmd_parser.parse_args(args)
        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), 2)
        mock_request.assert_called_once_with(
            test_cli20.end_url(path % my_id, query), 'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn('bytes_in', _str)
        self.assertIn('1234', _str)
        self.assertIn('bytes_out', _str)
        self.assertIn('4321', _str)

    def test_get_loadbalancer_statuses(self):
        # lbaas-loadbalancer-status test_id.
        resource = 'loadbalancer'
        cmd = lb.RetrieveLoadBalancerStatus(test_cli20.MyApp(sys.stdout), None)
        my_id = self.test_id
        args = [my_id]

        expected_res = {'statuses': {'operating_status': 'ONLINE',
                                     'provisioning_status': 'ACTIVE'}}

        resstr = self.client.serialize(expected_res)

        path = getattr(self.client, "lbaas_loadbalancer_path_status")
        return_tup = (test_cli20.MyResp(200), resstr)

        cmd_parser = cmd.get_parser("test_" + resource)
        parsed_args = cmd_parser.parse_args(args)
        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path % my_id), 'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn('operating_status', _str)
        self.assertIn('ONLINE', _str)
        self.assertIn('provisioning_status', _str)
        self.assertIn('ACTIVE', _str)



# Copyright (c) 2017 Juniper networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.cli import format_columns
from osc_lib.cli import parseractions
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc.v2.networking_bgpvpn import constants
from neutronclient.osc.v2.networking_bgpvpn import resource_association

LOG = logging.getLogger(__name__)


class BgpvpnPortAssoc(object):
    _assoc_res_name = constants.PORT_RESOURCE_NAME
    _resource = constants.PORT_ASSOC
    _resource_plural = constants.PORT_ASSOCS

    _attr_map = (
        ('id', 'ID', column_util.LIST_BOTH),
        ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
        ('%s_id' % _assoc_res_name, '%s ID' % _assoc_res_name.capitalize(),
         column_util.LIST_BOTH),
        ('prefix_routes', 'Prefix Routes (BGP LOCAL_PREF)',
         column_util.LIST_LONG_ONLY),
        ('bgpvpn_routes', 'BGP VPN Routes (BGP LOCAL_PREF)',
         column_util.LIST_LONG_ONLY),
        ('advertise_fixed_ips', "Advertise Port's Fixed IPs",
         column_util.LIST_LONG_ONLY),
    )
    _formatters = {
        'prefix_routes': format_columns.ListColumn,
        'bgpvpn_routes': format_columns.ListColumn,
    }

    def _transform_resource(self, data):
        """Transforms BGP VPN port association routes property

        That permits to easily format the command output with ListColumn
        formater and separate the two route types.

        {'routes':
            [
                {
                    'type': 'prefix',
                    'local_pref': 100,
                    'prefix': '8.8.8.0/27',
                },
                {
                    'type': 'prefix',
                    'local_pref': 42,
                    'prefix': '80.50.30.0/28',
                },
                {
                    'type': 'bgpvpn',
                    'local_pref': 50,
                    'bgpvpn': '157d72a9-9968-48e7-8087-6c9a9bc7a181',
                },
                {
                    'type': 'bgpvpn',
                    'bgpvpn': 'd5c7aaab-c7e8-48b3-85ca-a115c00d3603',
                },
            ],
        }

        to

        {
            'prefix_routes': [
                '8.8.8.0/27 (100)',
                '80.50.30.0/28 (42)',
            ],
            'bgpvpn_routes': [
                '157d72a9-9968-48e7-8087-6c9a9bc7a181 (50)',
                'd5c7aaab-c7e8-48b3-85ca-a115c00d3603',
            ],
        }
        """
        for route in data.get('routes', []):
            local_pref = ''
            if route.get('local_pref'):
                local_pref = ' (%d)' % route.get('local_pref')
            if route['type'] == 'prefix':
                data.setdefault('prefix_routes', []).append(
                    '%s%s' % (route['prefix'], local_pref)
                )
            elif route['type'] == 'bgpvpn':
                data.setdefault('bgpvpn_routes', []).append(
                    '%s%s' % (route['bgpvpn_id'], local_pref)
                )
            else:
                LOG.warning("Unknown route type %s (%s).", route['type'],
                            route)
        data.pop('routes', None)

    def _get_common_parser(self, parser):
        """Adds to parser arguments common to create, set and unset commands.

        :params ArgumentParser parser: argparse object contains all command's
                                       arguments
        """
        ADVERTISE_ROUTE = _("Fixed IPs of the port will be advertised to the "
                            "BGP VPN%s") % (
                                _(' (default)') if self._action == 'create'
                                else "")
        NOT_ADVERTISE_ROUTE = _("Fixed IPs of the port will not be advertised "
                                "to the BGP VPN")

        LOCAL_PREF_VALUE = _(". Optionally, can control the value of the BGP "
                             "LOCAL_PREF of the routes that will be "
                             "advertised")

        ADD_PREFIX_ROUTE = _("Add prefix route in CIDR notation%s") %\
            LOCAL_PREF_VALUE
        REMOVE_PREFIX_ROUTE = _("Remove prefix route in CIDR notation")
        REPEAT_PREFIX_ROUTE = _("repeat option for multiple prefix routes")

        ADD_BGVPVPN_ROUTE = _("Add BGP VPN route for route leaking%s") %\
            LOCAL_PREF_VALUE
        REMOVE_BGPVPN_ROUTE = _("Remove BGP VPN route")
        REPEAT_BGPVPN_ROUTE = _("repeat option for multiple BGP VPN routes")

        group_advertise_fixed_ips = parser.add_mutually_exclusive_group()
        group_advertise_fixed_ips.add_argument(
            '--advertise-fixed-ips',
            action='store_true',
            help=NOT_ADVERTISE_ROUTE if self._action == 'unset'
            else ADVERTISE_ROUTE,
        )
        group_advertise_fixed_ips.add_argument(
            '--no-advertise-fixed-ips',
            action='store_true',
            help=ADVERTISE_ROUTE if self._action == 'unset'
            else NOT_ADVERTISE_ROUTE,
        )

        if self._action in ['create', 'set']:
            parser.add_argument(
                '--prefix-route',
                metavar="prefix=<cidr>[,local_pref=<integer>]",
                dest='prefix_routes',
                action=parseractions.MultiKeyValueAction,
                required_keys=['prefix'],
                optional_keys=['local_pref'],
                help="%s (%s)" % (ADD_PREFIX_ROUTE, REPEAT_PREFIX_ROUTE),
            )
            parser.add_argument(
                '--bgpvpn-route',
                metavar="bgpvpn=<BGP VPN ID or name>[,local_pref=<integer>]",
                dest='bgpvpn_routes',
                action=parseractions.MultiKeyValueAction,
                required_keys=['bgpvpn'],
                optional_keys=['local_pref'],
                help="%s (%s)" % (ADD_BGVPVPN_ROUTE, REPEAT_BGPVPN_ROUTE),
            )
        else:
            parser.add_argument(
                '--prefix-route',
                metavar="<cidr>",
                dest='prefix_routes',
                action='append',
                help="%s (%s)" % (REMOVE_PREFIX_ROUTE, REPEAT_PREFIX_ROUTE),
            )
            parser.add_argument(
                '--bgpvpn-route',
                metavar="<BGP VPN ID or name>",
                dest='bgpvpn_routes',
                action='append',
                help="%s (%s)" % (REMOVE_BGPVPN_ROUTE, REPEAT_BGPVPN_ROUTE),
            )
        if self._action != 'create':
            parser.add_argument(
                '--no-prefix-route' if self._action == 'set' else
                '--all-prefix-routes',
                dest='purge_prefix_route',
                action='store_true',
                help=_('Empty prefix route list'),
            )
            parser.add_argument(
                '--no-bgpvpn-route' if self._action == 'set' else
                '--all-bgpvpn-routes',
                dest='purge_bgpvpn_route',
                action='store_true',
                help=_('Empty BGP VPN route list'),
            )

    def _args2body(self, bgpvpn_id, args):
        client = self.app.client_manager.neutronclient
        attrs = {}

        if self._action != 'create':
            assoc = client.find_resource_by_id(
                self._resource,
                args.resource_association_id,
                cmd_resource='bgpvpn_%s_assoc' % self._assoc_res_name,
                parent_id=bgpvpn_id)
        else:
            assoc = {'routes': []}

        if args.advertise_fixed_ips:
            attrs['advertise_fixed_ips'] = self._action != 'unset'
        elif args.no_advertise_fixed_ips:
            attrs['advertise_fixed_ips'] = self._action == 'unset'

        prefix_routes = None
        if 'purge_prefix_route' in args and args.purge_prefix_route:
            prefix_routes = []
        else:
            prefix_routes = {r['prefix']: r.get('local_pref')
                             for r in assoc['routes']
                             if r['type'] == 'prefix'}
            if args.prefix_routes:
                if self._action in ['create', 'set']:
                    prefix_routes.update({r['prefix']: r.get('local_pref')
                                          for r in args.prefix_routes})
                elif self._action == 'unset':
                    for prefix in args.prefix_routes:
                        prefix_routes.pop(prefix, None)

        bgpvpn_routes = None
        if 'purge_bgpvpn_route' in args and args.purge_bgpvpn_route:
            bgpvpn_routes = []
        else:
            bgpvpn_routes = {r['bgpvpn_id']: r.get('local_pref')
                             for r in assoc['routes']
                             if r['type'] == 'bgpvpn'}
            if args.bgpvpn_routes:
                if self._action == 'unset':
                    routes = [
                        {'bgpvpn': bgpvpn} for bgpvpn in args.bgpvpn_routes
                    ]
                else:
                    routes = args.bgpvpn_routes
                args_bgpvpn_routes = {
                    client.find_resource(constants.BGPVPN, r['bgpvpn'])['id']:
                        r.get('local_pref')
                    for r in routes
                }
                if self._action in ['create', 'set']:
                    bgpvpn_routes.update(args_bgpvpn_routes)
                elif self._action == 'unset':
                    for bgpvpn_id in args_bgpvpn_routes:
                        bgpvpn_routes.pop(bgpvpn_id, None)

        if prefix_routes is not None and not prefix_routes:
            attrs.setdefault('routes', [])
        elif prefix_routes is not None:
            for prefix, local_pref in prefix_routes.items():
                route = {
                    'type': 'prefix',
                    'prefix': prefix,
                }
                if local_pref:
                    route['local_pref'] = int(local_pref)
                attrs.setdefault('routes', []).append(route)
        if bgpvpn_routes is not None and not bgpvpn_routes:
            attrs.setdefault('routes', [])
        elif bgpvpn_routes is not None:
            for bgpvpn_id, local_pref in bgpvpn_routes.items():
                route = {
                    'type': 'bgpvpn',
                    'bgpvpn_id': bgpvpn_id,
                }
                if local_pref:
                    route['local_pref'] = int(local_pref)
                attrs.setdefault('routes', []).append(route)

        return {self._resource: attrs}


class CreateBgpvpnPortAssoc(BgpvpnPortAssoc,
                            resource_association.CreateBgpvpnResAssoc):
    _description = _("Create a BGP VPN port association")


class SetBgpvpnPortAssoc(BgpvpnPortAssoc,
                         resource_association.SetBgpvpnResAssoc):
    _description = _("Set BGP VPN port association properties")


class UnsetBgpvpnPortAssoc(BgpvpnPortAssoc,
                           resource_association.UnsetBgpvpnResAssoc):
    _description = _("Unset BGP VPN port association properties")


class DeleteBgpvpnPortAssoc(BgpvpnPortAssoc,
                            resource_association.DeleteBgpvpnResAssoc):
    _description = _("Delete a BGP VPN port association(s) for a given BGP "
                     "VPN")


class ListBgpvpnPortAssoc(BgpvpnPortAssoc,
                          resource_association.ListBgpvpnResAssoc):
    _description = _("List BGP VPN port associations for a given BGP VPN")


class ShowBgpvpnPortAssoc(BgpvpnPortAssoc,
                          resource_association.ShowBgpvpnResAssoc):
    _description = _("Show information of a given BGP VPN port association")



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.vpn import ipsec_site_connection
from neutronclient.tests.unit import test_cli20


class CLITestV20IPsecSiteConnectionJSON(test_cli20.CLITestV20Base):

    # TODO(pcm): Remove, once peer-cidr is deprecated completely
    def test_create_ipsec_site_connection_all_params_using_peer_cidrs(self):
        # ipsecsite-connection-create all params using peer CIDRs.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.CreateIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        tenant_id = 'mytenant_id'
        name = 'connection1'
        my_id = 'my_id'
        peer_address = '192.168.2.10'
        peer_id = '192.168.2.10'
        psk = 'abcd'
        mtu = '1500'
        initiator = 'bi-directional'
        vpnservice_id = 'vpnservice_id'
        ikepolicy_id = 'ikepolicy_id'
        ipsecpolicy_id = 'ipsecpolicy_id'
        peer_cidrs = ['192.168.3.0/24', '192.168.2.0/24']
        admin_state = True
        description = 'my-vpn-connection'
        dpd = 'action=restart,interval=30,timeout=120'

        args = ['--tenant-id', tenant_id,
                '--peer-address', peer_address, '--peer-id', peer_id,
                '--psk', psk, '--initiator', initiator,
                '--vpnservice-id', vpnservice_id,
                '--ikepolicy-id', ikepolicy_id, '--name', name,
                '--ipsecpolicy-id', ipsecpolicy_id, '--mtu', mtu,
                '--description', description,
                '--peer-cidr', '192.168.3.0/24',
                '--peer-cidr', '192.168.2.0/24',
                '--dpd', dpd]

        position_names = ['name', 'tenant_id', 'admin_state_up',
                          'peer_address', 'peer_id', 'peer_cidrs',
                          'psk', 'mtu', 'initiator', 'description',
                          'vpnservice_id', 'ikepolicy_id',
                          'ipsecpolicy_id']

        position_values = [name, tenant_id, admin_state, peer_address,
                           peer_id, peer_cidrs, psk, mtu,
                           initiator, description,
                           vpnservice_id, ikepolicy_id, ipsecpolicy_id]
        extra_body = {
            'dpd': {
                'action': 'restart',
                'interval': 30,
                'timeout': 120,
            },
        }

        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   extra_body=extra_body)

    def test_create_ipsec_site_conn_all_params(self):
        # ipsecsite-connection-create all params using endpoint groups.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.CreateIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        tenant_id = 'mytenant_id'
        name = 'connection1'
        my_id = 'my_id'
        peer_address = '192.168.2.10'
        peer_id = '192.168.2.10'
        psk = 'abcd'
        mtu = '1500'
        initiator = 'bi-directional'
        vpnservice_id = 'vpnservice_id'
        ikepolicy_id = 'ikepolicy_id'
        ipsecpolicy_id = 'ipsecpolicy_id'
        local_ep_group = 'local-epg'
        peer_ep_group = 'peer-epg'
        admin_state = True
        description = 'my-vpn-connection'
        dpd = 'action=restart,interval=30,timeout=120'

        args = ['--tenant-id', tenant_id,
                '--peer-address', peer_address, '--peer-id', peer_id,
                '--psk', psk, '--initiator', initiator,
                '--vpnservice-id', vpnservice_id,
                '--ikepolicy-id', ikepolicy_id, '--name', name,
                '--ipsecpolicy-id', ipsecpolicy_id, '--mtu', mtu,
                '--description', description,
                '--local-ep-group', local_ep_group,
                '--peer-ep-group', peer_ep_group,
                '--dpd', dpd]

        position_names = ['name', 'tenant_id', 'admin_state_up',
                          'peer_address', 'peer_id', 'psk', 'mtu',
                          'local_ep_group_id', 'peer_ep_group_id',
                          'initiator', 'description',
                          'vpnservice_id', 'ikepolicy_id',
                          'ipsecpolicy_id']

        position_values = [name, tenant_id, admin_state, peer_address,
                           peer_id, psk, mtu, local_ep_group,
                           peer_ep_group, initiator, description,
                           vpnservice_id, ikepolicy_id, ipsecpolicy_id]
        extra_body = {
            'dpd': {
                'action': 'restart',
                'interval': 30,
                'timeout': 120,
            },
        }

        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   extra_body=extra_body)

    def test_create_ipsec_site_connection_with_limited_params(self):
        # ipsecsite-connection-create with limited params.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.CreateIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        tenant_id = 'mytenant_id'
        my_id = 'my_id'
        peer_address = '192.168.2.10'
        peer_id = '192.168.2.10'
        psk = 'abcd'
        mtu = '1500'
        initiator = 'bi-directional'
        vpnservice_id = 'vpnservice_id'
        ikepolicy_id = 'ikepolicy_id'
        ipsecpolicy_id = 'ipsecpolicy_id'
        local_ep_group = 'local-epg'
        peer_ep_group = 'peer-epg'
        admin_state = True

        args = ['--tenant-id', tenant_id,
                '--peer-address', peer_address,
                '--peer-id', peer_id,
                '--psk', psk,
                '--vpnservice-id', vpnservice_id,
                '--ikepolicy-id', ikepolicy_id,
                '--ipsecpolicy-id', ipsecpolicy_id,
                '--local-ep-group', local_ep_group,
                '--peer-ep-group', peer_ep_group]

        position_names = ['tenant_id', 'admin_state_up',
                          'peer_address', 'peer_id',
                          'local_ep_group_id', 'peer_ep_group_id',
                          'psk', 'mtu', 'initiator',
                          'vpnservice_id', 'ikepolicy_id',
                          'ipsecpolicy_id']

        position_values = [tenant_id, admin_state, peer_address, peer_id,
                           local_ep_group, peer_ep_group, psk, mtu, initiator,
                           vpnservice_id, ikepolicy_id, ipsecpolicy_id]

        self._test_create_resource(resource, cmd, None, my_id, args,
                                   position_names, position_values)

    def _test_create_failure(self, additional_args=None, expected_exc=None):
        # Helper to test failure of IPSec site-to-site creation failure.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.CreateIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        tenant_id = 'mytenant_id'
        my_id = 'my_id'
        peer_address = '192.168.2.10'
        peer_id = '192.168.2.10'
        psk = 'abcd'
        mtu = '1500'
        initiator = 'bi-directional'
        vpnservice_id = 'vpnservice_id'
        ikepolicy_id = 'ikepolicy_id'
        ipsecpolicy_id = 'ipsecpolicy_id'
        admin_state = True

        args = ['--tenant-id', tenant_id,
                '--peer-address', peer_address,
                '--peer-id', peer_id,
                '--psk', psk,
                '--vpnservice-id', vpnservice_id,
                '--ikepolicy-id', ikepolicy_id,
                '--ipsecpolicy-id', ipsecpolicy_id]
        if additional_args is not None:
            args += additional_args
        position_names = ['tenant_id', 'admin_state_up', 'peer_address',
                          'peer_id', 'psk', 'mtu', 'initiator',
                          'local_ep_group_id', 'peer_ep_group_id',
                          'vpnservice_id', 'ikepolicy_id', 'ipsecpolicy_id']

        position_values = [tenant_id, admin_state, peer_address, peer_id, psk,
                           mtu, initiator, None, None, vpnservice_id,
                           ikepolicy_id, ipsecpolicy_id]
        if not expected_exc:
            expected_exc = exceptions.CommandError
        self.assertRaises(expected_exc,
                          self._test_create_resource,
                          resource, cmd, None, my_id, args,
                          position_names, position_values)

    def test_fail_create_with_invalid_mtu(self):
        # ipsecsite-connection-create with invalid dpd values.
        bad_mtu = ['--mtu', '67']
        self._test_create_failure(bad_mtu)

    def test_fail_create_with_invalid_dpd_keys(self):
        bad_dpd_key = ['--dpd', 'act=restart,interval=30,time=120']
        self._test_create_failure(bad_dpd_key, SystemExit)

    def test_fail_create_with_invalid_dpd_values(self):
        bad_dpd_values = ['--dpd', 'action=hold,interval=30,timeout=-1']
        self._test_create_failure(bad_dpd_values)

    def test_fail_create_missing_endpoint_groups_or_cidr(self):
        # Must provide either endpoint groups or peer cidrs.
        self._test_create_failure()

    def test_fail_create_missing_peer_endpoint_group(self):
        # Fails if dont have both endpoint groups - missing peer.
        self._test_create_failure(['--local-ep-group', 'local-epg'])

    def test_fail_create_missing_local_endpoint_group(self):
        # Fails if dont have both endpoint groups - missing local.
        self._test_create_failure(['--peer-ep-group', 'peer-epg'])

    def test_fail_create_when_both_endpoints_and_peer_cidr(self):
        # Cannot intermix endpoint groups and peer CIDRs for create.
        additional_args = ['--local-ep-group', 'local-epg',
                           '--peer-ep-group', 'peer-epg',
                           '--peer-cidr', '10.2.0.0/24']
        self._test_create_failure(additional_args)

    def test_list_ipsec_site_connection(self):
        # ipsecsite-connection-list.
        resources = "ipsec_site_connections"
        cmd = ipsec_site_connection.ListIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        self._test_list_resources(resources, cmd, True)

    def test_list_ipsec_site_connection_pagination(self):
        # ipsecsite-connection-list.
        resources = "ipsec_site_connections"
        cmd = ipsec_site_connection.ListIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_ipsec_site_connection_sort(self):
        # ipsecsite-connection-list.
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "ipsec_site_connections"
        cmd = ipsec_site_connection.ListIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_ipsec_site_connection_limit(self):
        # ipsecsite-connection-list -P.
        resources = "ipsec_site_connections"
        cmd = ipsec_site_connection.ListIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_ipsec_site_connection(self):
        # ipsecsite-connection-delete my-id.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.DeleteIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_update_ipsec_site_connection(self):
        # ipsecsite-connection-update  myid --name Branch-new --tags a b.
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.UpdateIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'Branch-new',
                                    '--tags', 'a', 'b'],
                                   {'name': 'Branch-new',
                                    'tags': ['a', 'b'], })
        # ipsecsite-connection-update myid --mtu 69 --initiator response-only
        # --peer-id '192.168.2.11' --peer-ep-group 'update-grp'
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--mtu', '69',
                                    '--initiator', 'response-only',
                                    '--peer-id', '192.168.2.11',
                                    '--peer-ep-group', 'update-grp'],
                                   {'mtu': '69',
                                    'initiator': 'response-only',
                                    'peer_id': '192.168.2.11',
                                    'peer_ep_group_id': 'update-grp', },)

    def test_show_ipsec_site_connection_id(self):
        # ipsecsite-connection-show test_id."""
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.ShowIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_ipsec_site_connection_id_name(self):
        # ipsecsite-connection-show."""
        resource = 'ipsec_site_connection'
        cmd = ipsec_site_connection.ShowIPsecSiteConnection(
            test_cli20.MyApp(sys.stdout), None
        )
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])



# Copyright 2013 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.neutron.v2_0 import network
from neutronclient.neutron.v2_0 import router


PERFECT_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"


class AddNetworkToDhcpAgent(neutronV20.NeutronCommand):
    """Add a network to a DHCP agent."""

    def get_parser(self, prog_name):
        parser = super(AddNetworkToDhcpAgent, self).get_parser(prog_name)
        parser.add_argument(
            'dhcp_agent',
            metavar='DHCP_AGENT',
            help=_('ID of the DHCP agent.'))
        parser.add_argument(
            'network',
            metavar='NETWORK',
            help=_('Network to add.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _net_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'network', parsed_args.network)
        neutron_client.add_network_to_dhcp_agent(parsed_args.dhcp_agent,
                                                 {'network_id': _net_id})
        print(_('Added network %s to DHCP agent') % parsed_args.network,
              file=self.app.stdout)


class RemoveNetworkFromDhcpAgent(neutronV20.NeutronCommand):
    """Remove a network from a DHCP agent."""

    def get_parser(self, prog_name):
        parser = super(RemoveNetworkFromDhcpAgent, self).get_parser(prog_name)
        parser.add_argument(
            'dhcp_agent',
            metavar='DHCP_AGENT',
            help=_('ID of the DHCP agent.'))
        parser.add_argument(
            'network',
            metavar='NETWORK',
            help=_('Network to remove.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _net_id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'network', parsed_args.network)
        neutron_client.remove_network_from_dhcp_agent(
            parsed_args.dhcp_agent, _net_id)
        print(_('Removed network %s from DHCP agent') % parsed_args.network,
              file=self.app.stdout)


class ListNetworksOnDhcpAgent(network.ListNetwork):
    """List the networks on a DHCP agent."""

    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListNetworksOnDhcpAgent,
                       self).get_parser(prog_name)
        parser.add_argument(
            'dhcp_agent',
            metavar='DHCP_AGENT',
            help=_('ID of the DHCP agent.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        data = neutron_client.list_networks_on_dhcp_agent(
            parsed_args.dhcp_agent, **search_opts)
        return data


class ListDhcpAgentsHostingNetwork(neutronV20.ListCommand):
    """List DHCP agents hosting a network."""

    resource = 'agent'
    _formatters = {}
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListDhcpAgentsHostingNetwork,
                       self).get_parser(prog_name)
        parser.add_argument(
            'network',
            metavar='NETWORK',
            help=_('Network to query.'))
        return parser

    def extend_list(self, data, parsed_args):
        for agent in data:
            agent['alive'] = ":-)" if agent['alive'] else 'xxx'

    def call_server(self, neutron_client, search_opts, parsed_args):
        _id = neutronV20.find_resourceid_by_name_or_id(neutron_client,
                                                       'network',
                                                       parsed_args.network)
        search_opts['network'] = _id
        data = neutron_client.list_dhcp_agent_hosting_networks(**search_opts)
        return data


class AddRouterToL3Agent(neutronV20.NeutronCommand):
    """Add a router to a L3 agent."""

    def get_parser(self, prog_name):
        parser = super(AddRouterToL3Agent, self).get_parser(prog_name)
        parser.add_argument(
            'l3_agent',
            metavar='L3_AGENT',
            help=_('ID of the L3 agent.'))
        parser.add_argument(
            'router',
            metavar='ROUTER',
            help=_('Router to add.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'router', parsed_args.router)
        neutron_client.add_router_to_l3_agent(parsed_args.l3_agent,
                                              {'router_id': _id})
        print(_('Added router %s to L3 agent') % parsed_args.router,
              file=self.app.stdout)


class RemoveRouterFromL3Agent(neutronV20.NeutronCommand):
    """Remove a router from a L3 agent."""

    def get_parser(self, prog_name):
        parser = super(RemoveRouterFromL3Agent, self).get_parser(prog_name)
        parser.add_argument(
            'l3_agent',
            metavar='L3_AGENT',
            help=_('ID of the L3 agent.'))
        parser.add_argument(
            'router',
            metavar='ROUTER',
            help=_('Router to remove.'))
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'router', parsed_args.router)
        neutron_client.remove_router_from_l3_agent(
            parsed_args.l3_agent, _id)
        print(_('Removed router %s from L3 agent') % parsed_args.router,
              file=self.app.stdout)


class ListRoutersOnL3Agent(neutronV20.ListCommand):
    """List the routers on a L3 agent."""

    _formatters = {'external_gateway_info':
                   router._format_external_gateway_info}
    list_columns = ['id', 'name', 'external_gateway_info']
    resource = 'router'
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListRoutersOnL3Agent,
                       self).get_parser(prog_name)
        parser.add_argument(
            'l3_agent',
            metavar='L3_AGENT',
            help=_('ID of the L3 agent to query.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        data = neutron_client.list_routers_on_l3_agent(
            parsed_args.l3_agent, **search_opts)
        return data


class ListL3AgentsHostingRouter(neutronV20.ListCommand):
    """List L3 agents hosting a router."""

    resource = 'agent'
    _formatters = {}
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListL3AgentsHostingRouter,
                       self).get_parser(prog_name)
        parser.add_argument('router',
                            metavar='ROUTER',
                            help=_('Router to query.'))
        return parser

    def extend_list(self, data, parsed_args):
        # Show the ha_state column only if the server responds with it,
        # as some plugins do not support HA routers.
        if any('ha_state' in agent for agent in data):
            if 'ha_state' not in self.list_columns:
                self.list_columns.append('ha_state')
        for agent in data:
            agent['alive'] = ":-)" if agent['alive'] else 'xxx'

    def call_server(self, neutron_client, search_opts, parsed_args):
        _id = neutronV20.find_resourceid_by_name_or_id(neutron_client,
                                                       'router',
                                                       parsed_args.router)
        search_opts['router'] = _id
        data = neutron_client.list_l3_agent_hosting_routers(**search_opts)
        return data


class ListPoolsOnLbaasAgent(neutronV20.ListCommand):
    """List the pools on a loadbalancer agent."""

    list_columns = ['id', 'name', 'lb_method', 'protocol',
                    'admin_state_up', 'status']
    resource = 'pool'
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListPoolsOnLbaasAgent, self).get_parser(prog_name)
        parser.add_argument(
            'lbaas_agent',
            metavar='LBAAS_AGENT',
            help=_('ID of the loadbalancer agent to query.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        data = neutron_client.list_pools_on_lbaas_agent(
            parsed_args.lbaas_agent, **search_opts)
        return data


class GetLbaasAgentHostingPool(neutronV20.ListCommand):
    """Get loadbalancer agent hosting a pool.

    Deriving from ListCommand though server will return only one agent
    to keep common output format for all agent schedulers
    """

    resource = 'agent'
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(GetLbaasAgentHostingPool,
                       self).get_parser(prog_name)
        parser.add_argument('pool',
                            metavar='POOL',
                            help=_('Pool to query.'))
        return parser

    def extend_list(self, data, parsed_args):
        for agent in data:
            agent['alive'] = ":-)" if agent['alive'] else 'xxx'

    def call_server(self, neutron_client, search_opts, parsed_args):
        _id = neutronV20.find_resourceid_by_name_or_id(neutron_client,
                                                       'pool',
                                                       parsed_args.pool)
        search_opts['pool'] = _id
        agent = neutron_client.get_lbaas_agent_hosting_pool(**search_opts)
        data = {'agents': [agent['agent']]}
        return data


class ListLoadBalancersOnLbaasAgent(neutronV20.ListCommand):
    """List the loadbalancers on a loadbalancer v2 agent."""

    list_columns = ['id', 'name', 'admin_state_up', 'provisioning_status']
    resource = 'loadbalancer'
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListLoadBalancersOnLbaasAgent, self).get_parser(
            prog_name)
        parser.add_argument(
            'lbaas_agent',
            metavar='LBAAS_AGENT',
            help=_('ID of the loadbalancer agent to query.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        data = neutron_client.list_loadbalancers_on_lbaas_agent(
            parsed_args.lbaas_agent, **search_opts)
        return data


class GetLbaasAgentHostingLoadBalancer(neutronV20.ListCommand):
    """Get lbaas v2 agent hosting a loadbalancer.

    Deriving from ListCommand though server will return only one agent
    to keep common output format for all agent schedulers
    """

    resource = 'agent'
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(GetLbaasAgentHostingLoadBalancer,
                       self).get_parser(prog_name)
        parser.add_argument('loadbalancer',
                            metavar='LOADBALANCER',
                            help=_('LoadBalancer to query.'))
        return parser

    def extend_list(self, data, parsed_args):
        for agent in data:
            agent['alive'] = ":-)" if agent['alive'] else 'xxx'

    def call_server(self, neutron_client, search_opts, parsed_args):
        _id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'loadbalancer', parsed_args.loadbalancer)
        search_opts['loadbalancer'] = _id
        agent = neutron_client.get_lbaas_agent_hosting_loadbalancer(
            **search_opts)
        data = {'agents': [agent['agent']]}
        return data



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""
Command-line interface to the Neutron APIs
"""

import argparse
import inspect
import itertools
import logging
import os
import sys

from keystoneauth1 import session
import os_client_config
from oslo_utils import encodeutils
from oslo_utils import netutils

from cliff import app
from cliff import command
from cliff import commandmanager

from neutronclient._i18n import _
from neutronclient.common import clientmanager
from neutronclient.common import exceptions as exc
from neutronclient.common import extension as client_extension
from neutronclient.neutron.v2_0 import subnet
from neutronclient.version import __version__


VERSION = '2.0'
NEUTRON_API_VERSION = '2.0'

NAMESPACE_MAP = {NEUTRON_API_VERSION: 'neutron.cli.v2'}


def run_command(cmd, cmd_parser, sub_argv):
    _argv = sub_argv
    index = -1
    values_specs = []
    if '--' in sub_argv:
        index = sub_argv.index('--')
        _argv = sub_argv[:index]
        values_specs = sub_argv[index:]
    known_args, _values_specs = cmd_parser.parse_known_args(_argv)
    if(isinstance(cmd, subnet.CreateSubnet) and not known_args.cidr):
        cidr = get_first_valid_cidr(_values_specs)
        if cidr:
            known_args.cidr = cidr
            _values_specs.remove(cidr)
    cmd.values_specs = (index == -1 and _values_specs or values_specs)
    return cmd.run(known_args)


def get_first_valid_cidr(value_specs):
    # Bug 1442771, argparse does not allow optional positional parameter
    # to be separated from previous positional parameter.
    # When cidr was separated from network, the value will not be able
    # to be parsed into known_args, but saved to _values_specs instead.
    for value in value_specs:
        if netutils.is_valid_cidr(value):
            return value


def env(*_vars, **kwargs):
    """Search for the first defined of possibly many env vars.

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.

    """
    for v in _vars:
        value = os.environ.get(v, None)
        if value:
            return value
    return kwargs.get('default', '')


def check_non_negative_int(value):
    try:
        value = int(value)
    except ValueError:
        raise argparse.ArgumentTypeError(_("invalid int value: %r") % value)
    if value < 0:
        raise argparse.ArgumentTypeError(_("input value %d is negative") %
                                         value)
    return value


COMMANDS = {}


# NOTE(amotoki): This is only to provide compatibility
# to existing neutron CLI extensions. See bug 1706573 for detail.
def _set_commands_dict_for_compat(apiversion, command_manager):
    global COMMANDS
    COMMANDS = {apiversion: dict((cmd, command_manager.find_command([cmd])[0])
                                 for cmd in command_manager.commands)}


class BashCompletionCommand(command.Command):
    """Prints all of the commands and options for bash-completion."""

    def take_action(self, parsed_args):
        pass


class HelpAction(argparse.Action):
    """Print help message including sub-commands

    Provide a custom action so the -h and --help options
    to the main app will print a list of the commands.

    The commands are determined by checking the CommandManager
    instance, passed in as the "default" value for the action.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        outputs = []
        max_len = 0
        app = self.default
        parser.print_help(app.stdout)
        app.stdout.write(_('\nCommands for API v%s:\n') % app.api_version)
        command_manager = app.command_manager
        for name, ep in sorted(command_manager):
            factory = ep.load()
            cmd = factory(self, None)
            one_liner = cmd.get_description().split('\n')[0]
            outputs.append((name, one_liner))
            max_len = max(len(name), max_len)
        for (name, one_liner) in outputs:
            app.stdout.write('  %s  %s\n' % (name.ljust(max_len), one_liner))
        sys.exit(0)


class NeutronShell(app.App):

    # verbose logging levels
    WARNING_LEVEL = 0
    INFO_LEVEL = 1
    DEBUG_LEVEL = 2
    CONSOLE_MESSAGE_FORMAT = '%(message)s'
    DEBUG_MESSAGE_FORMAT = '%(levelname)s: %(name)s %(message)s'
    log = logging.getLogger(__name__)

    def __init__(self, apiversion):
        namespace = NAMESPACE_MAP[apiversion]
        description = (__doc__.strip() +
                       " (neutron CLI version: %s)" % __version__)
        super(NeutronShell, self).__init__(
            description=description,
            version=VERSION,
            command_manager=commandmanager.CommandManager(namespace), )

        self._register_extensions(VERSION)

        # Pop the 'complete' to correct the outputs of 'neutron help'.
        self.command_manager.commands.pop('complete')

        # This is instantiated in initialize_app() only when using
        # password flow auth
        self.auth_client = None
        self.api_version = apiversion

        _set_commands_dict_for_compat(apiversion, self.command_manager)

    def build_option_parser(self, description, version):
        """Return an argparse option parser for this application.

        Subclasses may override this method to extend
        the parser with more global options.

        :param description: full description of the application
        :paramtype description: str
        :param version: version number for the application
        :paramtype version: str
        """
        parser = argparse.ArgumentParser(
            description=description,
            add_help=False, )
        parser.add_argument(
            '--version',
            action='version',
            version=__version__, )
        parser.add_argument(
            '-v', '--verbose', '--debug',
            action='count',
            dest='verbose_level',
            default=self.DEFAULT_VERBOSE_LEVEL,
            help=_('Increase verbosity of output and show tracebacks on'
                   ' errors. You can repeat this option.'))
        parser.add_argument(
            '-q', '--quiet',
            action='store_const',
            dest='verbose_level',
            const=0,
            help=_('Suppress output except warnings and errors.'))
        parser.add_argument(
            '-h', '--help',
            action=HelpAction,
            nargs=0,
            default=self,  # tricky
            help=_("Show this help message and exit."))
        parser.add_argument(
            '-r', '--retries',
            metavar="NUM",
            type=check_non_negative_int,
            default=0,
            help=_("How many times the request to the Neutron server should "
                   "be retried if it fails. Defaults to 0."))
        # FIXME(bklei): this method should come from keystoneauth1
        self._append_global_identity_args(parser)

        return parser

    def _append_global_identity_args(self, parser):
        # FIXME(bklei): these are global identity (Keystone) arguments which
        # should be consistent and shared by all service clients. Therefore,
        # they should be provided by keystoneauth1. We will need to
        # refactor this code once this functionality is available in
        # keystoneauth1.
        #
        # Note: At that time we'll need to decide if we can just abandon
        #       the deprecated args (--service-type and --endpoint-type).

        parser.add_argument(
            '--os-service-type', metavar='<os-service-type>',
            default=env('OS_NETWORK_SERVICE_TYPE', default='network'),
            help=_('Defaults to env[OS_NETWORK_SERVICE_TYPE] or "network".'))

        parser.add_argument(
            '--os-endpoint-type', metavar='<os-endpoint-type>',
            default=env('OS_ENDPOINT_TYPE', default='public'),
            help=_('Defaults to env[OS_ENDPOINT_TYPE] or "public".'))

        # FIXME(bklei): --service-type is deprecated but kept in for
        # backward compatibility.
        parser.add_argument(
            '--service-type', metavar='<service-type>',
            default=env('OS_NETWORK_SERVICE_TYPE', default='network'),
            help=_('DEPRECATED! Use --os-service-type.'))

        # FIXME(bklei): --endpoint-type is deprecated but kept in for
        # backward compatibility.
        parser.add_argument(
            '--endpoint-type', metavar='<endpoint-type>',
            default=env('OS_ENDPOINT_TYPE', default='public'),
            help=_('DEPRECATED! Use --os-endpoint-type.'))

        parser.add_argument(
            '--os-auth-strategy', metavar='<auth-strategy>',
            default=env('OS_AUTH_STRATEGY', default='keystone'),
            help=_('DEPRECATED! Only keystone is supported.'))

        parser.add_argument(
            '--os_auth_strategy',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-cloud', metavar='<cloud>',
            help=_('Defaults to env[OS_CLOUD].'))

        parser.add_argument(
            '--os-auth-url', metavar='<auth-url>',
            help=_('Authentication URL, defaults to env[OS_AUTH_URL].'))
        parser.add_argument(
            '--os_auth_url',
            help=argparse.SUPPRESS)

        project_name_group = parser.add_mutually_exclusive_group()
        project_name_group.add_argument(
            '--os-tenant-name', metavar='<auth-tenant-name>',
            help=_('Authentication tenant name, defaults to '
                   'env[OS_TENANT_NAME].'))
        project_name_group.add_argument(
            '--os-project-name',
            metavar='<auth-project-name>',
            help=_('Another way to specify tenant name. '
                   'This option is mutually exclusive with '
                   ' --os-tenant-name. '
                   'Defaults to env[OS_PROJECT_NAME].'))

        parser.add_argument(
            '--os_tenant_name',
            help=argparse.SUPPRESS)

        project_id_group = parser.add_mutually_exclusive_group()
        project_id_group.add_argument(
            '--os-tenant-id', metavar='<auth-tenant-id>',
            help=_('Authentication tenant ID, defaults to '
                   'env[OS_TENANT_ID].'))
        project_id_group.add_argument(
            '--os-project-id',
            metavar='<auth-project-id>',
            help=_('Another way to specify tenant ID. '
                   'This option is mutually exclusive with '
                   ' --os-tenant-id. '
                   'Defaults to env[OS_PROJECT_ID].'))

        parser.add_argument(
            '--os-username', metavar='<auth-username>',
            help=_('Authentication username, defaults to env[OS_USERNAME].'))
        parser.add_argument(
            '--os_username',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-user-id', metavar='<auth-user-id>',
            help=_('Authentication user ID (Env: OS_USER_ID)'))

        parser.add_argument(
            '--os_user_id',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-user-domain-id',
            metavar='<auth-user-domain-id>',
            help=_('OpenStack user domain ID. '
                   'Defaults to env[OS_USER_DOMAIN_ID].'))

        parser.add_argument(
            '--os_user_domain_id',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-user-domain-name',
            metavar='<auth-user-domain-name>',
            help=_('OpenStack user domain name. '
                   'Defaults to env[OS_USER_DOMAIN_NAME].'))

        parser.add_argument(
            '--os_user_domain_name',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os_project_id',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os_project_name',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-project-domain-id',
            metavar='<auth-project-domain-id>',
            help=_('Defaults to env[OS_PROJECT_DOMAIN_ID].'))

        parser.add_argument(
            '--os-project-domain-name',
            metavar='<auth-project-domain-name>',
            help=_('Defaults to env[OS_PROJECT_DOMAIN_NAME].'))

        parser.add_argument(
            '--os-cert',
            metavar='<certificate>',
            help=_("Path of certificate file to use in SSL "
                   "connection. This file can optionally be "
                   "prepended with the private key. Defaults "
                   "to env[OS_CERT]."))

        parser.add_argument(
            '--os-cacert',
            metavar='<ca-certificate>',
            help=_("Specify a CA bundle file to use in "
                   "verifying a TLS (https) server certificate. "
                   "Defaults to env[OS_CACERT]."))

        parser.add_argument(
            '--os-key',
            metavar='<key>',
            help=_("Path of client key to use in SSL "
                   "connection. This option is not necessary "
                   "if your key is prepended to your certificate "
                   "file. Defaults to env[OS_KEY]."))

        parser.add_argument(
            '--os-password', metavar='<auth-password>',
            help=_('Authentication password, defaults to env[OS_PASSWORD].'))
        parser.add_argument(
            '--os_password',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-region-name', metavar='<auth-region-name>',
            help=_('Authentication region name, defaults to '
                   'env[OS_REGION_NAME].'))
        parser.add_argument(
            '--os_region_name',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--os-token', metavar='<token>',
            help=_('Authentication token, defaults to env[OS_TOKEN].'))
        parser.add_argument(
            '--os_token',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--http-timeout', metavar='<seconds>',
            default=env('OS_NETWORK_TIMEOUT', default=None), type=float,
            help=_('Timeout in seconds to wait for an HTTP response. Defaults '
                   'to env[OS_NETWORK_TIMEOUT] or None if not specified.'))

        parser.add_argument(
            '--os-url', metavar='<url>',
            help=_('Defaults to env[OS_URL].'))
        parser.add_argument(
            '--os_url',
            help=argparse.SUPPRESS)

        parser.add_argument(
            '--insecure',
            action='store_true',
            default=env('NEUTRONCLIENT_INSECURE', default=False),
            help=_("Explicitly allow neutronclient to perform \"insecure\" "
                   "SSL (https) requests. The server's certificate will "
                   "not be verified against any certificate authorities. "
                   "This option should be used with caution."))

    def _bash_completion(self):
        """Prints all of the commands and options for bash-completion."""
        commands = set()
        options = set()
        for option, _action in self.parser._option_string_actions.items():
            options.add(option)
        for _name, _command in self.command_manager:
            commands.add(_name)
            cmd_factory = _command.load()
            cmd = cmd_factory(self, None)
            cmd_parser = cmd.get_parser('')
            for option, _action in cmd_parser._option_string_actions.items():
                options.add(option)
        print(' '.join(commands | options))

    def _register_extensions(self, version):
        for name, module in itertools.chain(
                client_extension._discover_via_entry_points()):
            self._extend_shell_commands(name, module, version)

    def _extend_shell_commands(self, name, module, version):
        classes = inspect.getmembers(module, inspect.isclass)
        for cls_name, cls in classes:
            if (issubclass(cls, client_extension.NeutronClientExtension) and
                    hasattr(cls, 'shell_command')):
                cmd = cls.shell_command
                if hasattr(cls, 'versions'):
                    if version not in cls.versions:
                        continue
                try:
                    name_prefix = "[%s]" % name
                    cls.__doc__ = ("%s %s" % (name_prefix, cls.__doc__) if
                                   cls.__doc__ else name_prefix)
                    self.command_manager.add_command(cmd, cls)
                except TypeError:
                    pass

    def run(self, argv):
        """Equivalent to the main program for the application.

        :param argv: input arguments and options
        :paramtype argv: list of str
        """
        try:
            index = 0
            command_pos = -1
            help_pos = -1
            help_command_pos = -1
            for arg in argv:
                if arg == 'bash-completion' and help_command_pos == -1:
                    self._bash_completion()
                    return 0
                if arg in ('-h', '--help'):
                    if help_pos == -1:
                        help_pos = index
                # self.command_manager.commands contains 'help',
                # so we need to check this first.
                elif arg == 'help':
                    if help_command_pos == -1:
                        help_command_pos = index
                elif arg in self.command_manager.commands:
                    if command_pos == -1:
                        command_pos = index
                index = index + 1
            if command_pos > -1 and help_pos > command_pos:
                argv = ['help', argv[command_pos]]
            if help_command_pos > -1 and command_pos == -1:
                argv[help_command_pos] = '--help'
            self.options, remainder = self.parser.parse_known_args(argv)
            self.configure_logging()
            self.interactive_mode = not remainder
            self.initialize_app(remainder)
        except Exception as err:
            if self.options.verbose_level >= self.DEBUG_LEVEL:
                self.log.exception(err)
                raise
            else:
                self.log.error(err)
            return 1
        if self.interactive_mode:
            _argv = [sys.argv[0]]
            sys.argv = _argv
            return self.interact()
        return self.run_subcommand(remainder)

    def run_subcommand(self, argv):
        subcommand = self.command_manager.find_command(argv)
        cmd_factory, cmd_name, sub_argv = subcommand
        cmd = cmd_factory(self, self.options)
        try:
            self.prepare_to_run_command(cmd)
            full_name = (cmd_name
                         if self.interactive_mode
                         else ' '.join([self.NAME, cmd_name])
                         )
            cmd_parser = cmd.get_parser(full_name)
            return run_command(cmd, cmd_parser, sub_argv)
        except SystemExit:
            print(_("Try 'neutron help %s' for more information.") %
                  cmd_name, file=sys.stderr)
            raise
        except Exception as e:
            if self.options.verbose_level >= self.DEBUG_LEVEL:
                self.log.exception("%s", e)
                raise
            self.log.error("%s", e)
        return 1

    def authenticate_user(self):
        """Confirm user authentication

        Make sure the user has provided all of the authentication
        info we need.
        """
        cloud_config = os_client_config.OpenStackConfig().get_one_cloud(
            cloud=self.options.os_cloud, argparse=self.options,
            network_api_version=self.api_version,
            verify=not self.options.insecure)
        verify, cert = cloud_config.get_requests_verify_args()

        # TODO(singhj): Remove dependancy on HTTPClient
        # for the case of token-endpoint authentication

        # When using token-endpoint authentication legacy
        # HTTPClient will be used, otherwise SessionClient
        # will be used.
        if self.options.os_token and self.options.os_url:
            auth = None
            auth_session = None
        else:
            auth = cloud_config.get_auth()

            auth_session = session.Session(
                auth=auth, verify=verify, cert=cert,
                timeout=self.options.http_timeout)

        interface = self.options.os_endpoint_type or self.endpoint_type
        if interface.endswith('URL'):
            interface = interface[:-3]
        self.client_manager = clientmanager.ClientManager(
            retries=self.options.retries,
            raise_errors=False,
            session=auth_session,
            url=self.options.os_url,
            token=self.options.os_token,
            region_name=cloud_config.get_region_name(),
            api_version=cloud_config.get_api_version('network'),
            service_type=cloud_config.get_service_type('network'),
            service_name=cloud_config.get_service_name('network'),
            endpoint_type=interface,
            auth=auth,
            insecure=not verify,
            log_credentials=True)
        return

    def initialize_app(self, argv):
        """Global app init bits:

        * set up API versions
        * validate authentication info
        """

        super(NeutronShell, self).initialize_app(argv)

        # If the user is not asking for help, make sure they
        # have given us auth.
        cmd_name = None
        if argv:
            cmd_info = self.command_manager.find_command(argv)
            cmd_factory, cmd_name, sub_argv = cmd_info
        if self.interactive_mode or cmd_name != 'help':
            self.authenticate_user()

    def configure_logging(self):
        """Create logging handlers for any log output."""
        root_logger = logging.getLogger('')

        # Set up logging to a file
        root_logger.setLevel(logging.DEBUG)

        # Send higher-level messages to the console via stderr
        console = logging.StreamHandler(self.stderr)
        console_level = {self.WARNING_LEVEL: logging.WARNING,
                         self.INFO_LEVEL: logging.INFO,
                         self.DEBUG_LEVEL: logging.DEBUG,
                         }.get(self.options.verbose_level, logging.DEBUG)
        # The default log level is INFO, in this situation, set the
        # log level of the console to WARNING, to avoid displaying
        # useless messages. This equals using "--quiet"
        if console_level == logging.INFO:
            console.setLevel(logging.WARNING)
        else:
            console.setLevel(console_level)
        if logging.DEBUG == console_level:
            formatter = logging.Formatter(self.DEBUG_MESSAGE_FORMAT)
        else:
            formatter = logging.Formatter(self.CONSOLE_MESSAGE_FORMAT)
        logging.getLogger('iso8601.iso8601').setLevel(logging.WARNING)
        logging.getLogger('urllib3.connectionpool').setLevel(logging.WARNING)
        console.setFormatter(formatter)
        root_logger.addHandler(console)
        return


def main(argv=sys.argv[1:]):
    try:
        print(_("neutron CLI is deprecated and will be removed "
                "in the future. Use openstack CLI instead."), file=sys.stderr)
        return NeutronShell(NEUTRON_API_VERSION).run(
            list(map(encodeutils.safe_decode, argv)))
    except KeyboardInterrupt:
        print(_("... terminating neutron client"), file=sys.stderr)
        return 130
    except exc.NeutronClientException:
        return 1
    except Exception as e:
        print(e)
        return 1


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python
# Copyright 2012 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient.neutron.v2_0 import floatingip as fip
from neutronclient.tests.unit import test_cli20


class CLITestV20FloatingIpsJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['floatingip']

    def test_create_floatingip(self):
        # Create floatingip: fip1.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        args = [name, '--description', 'floats like a butterfly']
        position_names = ['floating_network_id']
        position_values = [name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='floats like a butterfly')

    def test_create_floatingip_and_port(self):
        # Create floatingip: fip1.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        pid = 'mypid'
        args = [name, '--port_id', pid]
        position_names = ['floating_network_id', 'port_id']
        position_values = [name, pid]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

        # Test dashed options
        args = [name, '--port-id', pid]
        position_names = ['floating_network_id', 'port_id']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_floatingip_and_port_and_address(self):
        # Create floatingip: fip1 with a given port and address.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        pid = 'mypid'
        addr = '10.0.0.99'
        args = [name, '--port_id', pid, '--fixed_ip_address', addr]
        position_names = ['floating_network_id', 'port_id', 'fixed_ip_address']
        position_values = [name, pid, addr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)
        # Test dashed options
        args = [name, '--port-id', pid, '--fixed-ip-address', addr]
        position_names = ['floating_network_id', 'port_id', 'fixed_ip_address']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_floatingip_with_ip_address_of_floating_ip(self):
        # Create floatingip: fip1 with a given IP address of floating IP.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        addr = '10.0.0.99'

        args = [name, '--floating-ip-address', addr]
        position_values = [name, addr]
        position_names = ['floating_network_id', 'floating_ip_address']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_floatingip_with_subnet_id(self):
        # Create floatingip: fip1 on a given subnet id.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        subnet_id = 'mysubnetid'

        args = [name, '--subnet', subnet_id]
        position_values = [name, subnet_id]
        position_names = ['floating_network_id', 'subnet_id']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_floatingip_with_subnet_id_and_port(self):
        # Create floatingip: fip1 on a given subnet id and port.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        pid = 'mypid'
        subnet_id = 'mysubnetid'

        args = [name, '--subnet', subnet_id, '--port-id', pid]
        position_values = [name, subnet_id, pid]
        position_names = ['floating_network_id', 'subnet_id', 'port_id']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_floatingip_with_dns_name_and_dns_domain(self):
        # Create floatingip: fip1 with dns name and dns domain.
        resource = 'floatingip'
        cmd = fip.CreateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        name = 'fip1'
        myid = 'myid'
        dns_name_name = 'my-floatingip'
        dns_domain_name = 'my-domain.org.'
        args = [name, '--dns-name', dns_name_name, '--dns-domain',
                dns_domain_name]
        position_names = ['floating_network_id', 'dns_name', 'dns_domain']
        position_values = [name, dns_name_name, dns_domain_name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_floatingips(self):
        # list floatingips: -D.
        resources = 'floatingips'
        cmd = fip.ListFloatingIP(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_floatingips_pagination(self):
        resources = 'floatingips'
        cmd = fip.ListFloatingIP(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_floatingips_sort(self):
        # list floatingips:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = 'floatingips'
        cmd = fip.ListFloatingIP(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_floatingips_limit(self):
        # list floatingips: -P.
        resources = 'floatingips'
        cmd = fip.ListFloatingIP(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_floatingip(self):
        # Delete floatingip: fip1.
        resource = 'floatingip'
        cmd = fip.DeleteFloatingIP(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_show_floatingip(self):
        # Show floatingip: --fields id.
        resource = 'floatingip'
        cmd = fip.ShowFloatingIP(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id'])

    def test_disassociate_ip(self):
        # Disassociate floating IP: myid.
        resource = 'floatingip'
        cmd = fip.DisassociateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        args = ['myid']
        self._test_update_resource(resource, cmd, 'myid',
                                   args, {"port_id": None}
                                   )

    def test_associate_ip(self):
        # Associate floating IP: myid portid.
        resource = 'floatingip'
        cmd = fip.AssociateFloatingIP(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'portid']
        self._test_update_resource(resource, cmd, 'myid',
                                   args, {"port_id": "portid"}
                                   )



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutronclient.osc.v2.sfc import sfc_flow_classifier
from neutronclient.tests.unit.osc.v2.sfc import fakes

get_id = 'neutronclient.osc.v2.sfc.sfc_flow_classifier._get_id'


def _get_id(client, id_or_name, resource):
    return id_or_name


class TestCreateSfcFlowClassifier(fakes.TestNeutronClientOSCV2):

    _fc = fakes.FakeSfcFlowClassifier.create_flow_classifier()

    columns = ('Description',
               'Destination IP',
               'Destination Port Range Max',
               'Destination Port Range Min',
               'Ethertype',
               'ID',
               'L7 Parameters',
               'Logical Destination Port',
               'Logical Source Port',
               'Name',
               'Project',
               'Protocol',
               'Source IP',
               'Source Port Range Max',
               'Source Port Range Min')

    def get_data(self):
        return (
            self._fc['description'],
            self._fc['destination_ip_prefix'],
            self._fc['destination_port_range_max'],
            self._fc['destination_port_range_min'],
            self._fc['ethertype'],
            self._fc['id'],
            self._fc['l7_parameters'],
            self._fc['logical_destination_port'],
            self._fc['logical_source_port'],
            self._fc['name'],
            self._fc['project_id'],
            self._fc['protocol'],
            self._fc['source_ip_prefix'],
            self._fc['source_port_range_max'],
            self._fc['source_port_range_min']
        )

    def setUp(self):
        super(TestCreateSfcFlowClassifier, self).setUp()
        mock.patch(get_id, new=_get_id).start()
        self.neutronclient.create_sfc_flow_classifier = mock.Mock(
            return_value={'flow_classifier': self._fc})
        self.data = self.get_data()

        # Get the command object to test
        self.cmd = sfc_flow_classifier.CreateSfcFlowClassifier(self.app,
                                                               self.namespace)

    def test_create_flow_classifier_default_options(self):
        arglist = [
            "--logical-source-port", self._fc['logical_source_port'],
            "--ethertype", self._fc['ethertype'],
            self._fc['name'],
        ]
        verifylist = [
            ('logical_source_port', self._fc['logical_source_port']),
            ('ethertype', self._fc['ethertype']),
            ('name', self._fc['name']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_sfc_flow_classifier.assert_called_once_with({
            'flow_classifier': {
                'name': self._fc['name'],
                'logical_source_port': self._fc['logical_source_port'],
                'ethertype': self._fc['ethertype']}
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_flow_classifier(self):
        arglist = [
            "--description", self._fc['description'],
            "--ethertype", self._fc['ethertype'],
            "--protocol", self._fc['protocol'],
            "--source-ip-prefix", self._fc['source_ip_prefix'],
            "--destination-ip-prefix", self._fc['destination_ip_prefix'],
            "--logical-source-port", self._fc['logical_source_port'],
            "--logical-destination-port", self._fc['logical_destination_port'],
            self._fc['name'],
            "--l7-parameters", 'url=path',
        ]

        param = 'url=path'

        verifylist = [
            ('description', self._fc['description']),
            ('name', self._fc['name']),
            ('ethertype', self._fc['ethertype']),
            ('protocol', self._fc['protocol']),
            ('source_ip_prefix', self._fc['source_ip_prefix']),
            ('destination_ip_prefix', self._fc['destination_ip_prefix']),
            ('logical_source_port', self._fc['logical_source_port']),
            ('logical_destination_port',
             self._fc['logical_destination_port']),
            ('l7_parameters', param)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))
        self.neutronclient.create_sfc_flow_classifier.assert_called_once_with({
            'flow_classifier': {
                'name': self._fc['name'],
                'description': self._fc['description'],
                'ethertype': self._fc['ethertype'],
                'protocol': self._fc['protocol'],
                'source_ip_prefix': self._fc['source_ip_prefix'],
                'destination_ip_prefix': self._fc['destination_ip_prefix'],
                'logical_source_port': self._fc['logical_source_port'],
                'logical_destination_port':
                    self._fc['logical_destination_port'],
                'l7_parameters': param
                }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteSfcFlowClassifier(fakes.TestNeutronClientOSCV2):

    _flow_classifier = \
        fakes.FakeSfcFlowClassifier.create_flow_classifiers(count=1)

    def setUp(self):
        super(TestDeleteSfcFlowClassifier, self).setUp()
        mock.patch(get_id, new=_get_id).start()
        self.neutronclient.delete_sfc_flow_classifier = mock.Mock(
            return_value=None)
        self.cmd = sfc_flow_classifier.DeleteSfcFlowClassifier(self.app,
                                                               self.namespace)

    def test_delete_flow_classifier(self):
        client = self.app.client_manager.neutronclient
        mock_flow_classifier_delete = client.delete_sfc_flow_classifier
        arglist = [
            self._flow_classifier[0]['id'],
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier[0]['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        mock_flow_classifier_delete.assert_called_once_with(
            self._flow_classifier[0]['id'])
        self.assertIsNone(result)


class TestSetSfcFlowClassifier(fakes.TestNeutronClientOSCV2):
    _flow_classifier = fakes.FakeSfcFlowClassifier.create_flow_classifier()
    _flow_classifier_name = _flow_classifier['name']
    _flow_classifier_id = _flow_classifier['id']

    def setUp(self):
        super(TestSetSfcFlowClassifier, self).setUp()
        mock.patch(get_id, new=_get_id).start()
        self.neutronclient.update_sfc_flow_classifier = mock.Mock(
            return_value=None)
        self.cmd = sfc_flow_classifier.SetSfcFlowClassifier(self.app,
                                                            self.namespace)

    def test_set_flow_classifier(self):
        client = self.app.client_manager.neutronclient
        mock_flow_classifier_update = client.update_sfc_flow_classifier
        arglist = [
            self._flow_classifier_name,
            '--name', 'name_updated',
            '--description', 'desc_updated'
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier_name),
            ('name', 'name_updated'),
            ('description', 'desc_updated'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {'flow_classifier': {
            'name': 'name_updated',
            'description': 'desc_updated'}}
        mock_flow_classifier_update.assert_called_once_with(
            self._flow_classifier_name, attrs)
        self.assertIsNone(result)


class TestShowSfcFlowClassifier(fakes.TestNeutronClientOSCV2):

    _fc = fakes.FakeSfcFlowClassifier.create_flow_classifier()
    data = (
        _fc['description'],
        _fc['destination_ip_prefix'],
        _fc['destination_port_range_max'],
        _fc['destination_port_range_min'],
        _fc['ethertype'],
        _fc['id'],
        _fc['l7_parameters'],
        _fc['logical_destination_port'],
        _fc['logical_source_port'],
        _fc['name'],
        _fc['project_id'],
        _fc['protocol'],
        _fc['source_ip_prefix'],
        _fc['source_port_range_max'],
        _fc['source_port_range_min']
    )
    _flow_classifier = {'flow_classifier': _fc}
    _flow_classifier_id = _fc['id']
    columns = ('Description',
               'Destination IP',
               'Destination Port Range Max',
               'Destination Port Range Min',
               'Ethertype',
               'ID',
               'L7 Parameters',
               'Logical Destination Port',
               'Logical Source Port',
               'Name',
               'Project',
               'Protocol',
               'Source IP',
               'Source Port Range Max',
               'Source Port Range Min')

    def setUp(self):
        super(TestShowSfcFlowClassifier, self).setUp()
        mock.patch(get_id, new=_get_id).start()
        self.neutronclient.show_sfc_flow_classifier = mock.Mock(
            return_value=self._flow_classifier
        )
        # Get the command object to test
        self.cmd = sfc_flow_classifier.ShowSfcFlowClassifier(self.app,
                                                             self.namespace)

    def test_show_flow_classifier(self):
        client = self.app.client_manager.neutronclient
        mock_flow_classifier_show = client.show_sfc_flow_classifier
        arglist = [
            self._flow_classifier_id,
        ]
        verifylist = [
            ('flow_classifier', self._flow_classifier_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        mock_flow_classifier_show.assert_called_once_with(
            self._flow_classifier_id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestListSfcFlowClassifier(fakes.TestNeutronClientOSCV2):

    _fc = fakes.FakeSfcFlowClassifier.create_flow_classifiers(count=1)

    columns = ('ID', 'Name', 'Summary')

    columns_long = ('ID', 'Name', 'Protocol', 'Ethertype', 'Source IP',
                    'Destination IP', 'Logical Source Port',
                    'Logical Destination Port', 'Source Port Range Min',
                    'Source Port Range Max', 'Destination Port Range Min',
                    'Destination Port Range Max', 'L7 Parameters',
                    'Description', 'Project')
    _flow_classifier = _fc[0]
    data = [
        _flow_classifier['id'],
        _flow_classifier['name'],
        _flow_classifier['protocol'],
        _flow_classifier['source_ip_prefix'],
        _flow_classifier['destination_ip_prefix'],
        _flow_classifier['logical_source_port'],
        _flow_classifier['logical_destination_port']
    ]
    data_long = [
        _flow_classifier['id'],
        _flow_classifier['name'],
        _flow_classifier['protocol'],
        _flow_classifier['ethertype'],
        _flow_classifier['source_ip_prefix'],
        _flow_classifier['destination_ip_prefix'],
        _flow_classifier['logical_source_port'],
        _flow_classifier['logical_destination_port'],
        _flow_classifier['source_port_range_min'],
        _flow_classifier['source_port_range_max'],
        _flow_classifier['destination_port_range_min'],
        _flow_classifier['destination_port_range_max'],
        _flow_classifier['l7_parameters'],
        _flow_classifier['description']
    ]

    _flow_classifier1 = {'flow_classifiers': _flow_classifier}
    _flow_classifier_id = _flow_classifier['id']

    def setUp(self):
        super(TestListSfcFlowClassifier, self).setUp()
        mock.patch(get_id, new=_get_id).start()
        self.neutronclient.list_sfc_flow_classifiers = mock.Mock(
            return_value={'flow_classifiers': self._fc}
        )
        # Get the command object to test
        self.cmd = sfc_flow_classifier.ListSfcFlowClassifier(self.app,
                                                             self.namespace)

    def test_list_flow_classifiers(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns = self.cmd.take_action(parsed_args)
        fcs = self.neutronclient \
            .list_sfc_flow_classifiers()['flow_classifiers']
        fc = fcs[0]
        data = [
            fc['id'],
            fc['name'],
            fc['protocol'],
            fc['source_ip_prefix'],
            fc['destination_ip_prefix'],
            fc['logical_source_port'],
            fc['logical_destination_port']
        ]
        self.assertEqual(list(self.columns), columns[0])
        self.assertEqual(self.data, data)

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        fcs = self.neutronclient \
            .list_sfc_flow_classifiers()['flow_classifiers']
        fc = fcs[0]
        data = [
            fc['id'],
            fc['name'],
            fc['protocol'],
            fc['ethertype'],
            fc['source_ip_prefix'],
            fc['destination_ip_prefix'],
            fc['logical_source_port'],
            fc['logical_destination_port'],
            fc['source_port_range_min'],
            fc['source_port_range_max'],
            fc['destination_port_range_min'],
            fc['destination_port_range_max'],
            fc['l7_parameters'],
            fc['description']
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns_long = self.cmd.take_action(parsed_args)[0]
        self.assertEqual(list(self.columns_long), columns_long)
        self.assertEqual(self.data_long, data)
